import com.haxepunk.Scene;
import com.haxepunk.graphics.Backdrop;
import com.haxepunk.graphics.Image;
import com.haxepunk.graphics.Text;
import com.haxepunk.utils.*;

import com.haxepunk.HXP;
import com.haxepunk.Entity;
import com.haxepunk.Sfx;

import flash.text.*;



class MainScene extends Scene {

	public var title: Entity;


	public var lighting: Lighting;
	public var player: Player;
	public var sky: Backdrop;
	public var surface1: Backdrop;
	public var surface2: Backdrop;

	public var cells: Map<String, Cell> = new Map<String, Cell>();
	public var fish: Int = 0;


	public var cargo: Text;
	public var money: Text;

	public var music:Sfx;


	public override function begin() {

		music = new Sfx("audio/music.mp3");
		music.play(1, 0, true);

		// title
		title = addGraphic( new Image("graphics/title.png"), -1, -140, 200 );

		
		sky = new Backdrop( "graphics/sky.png", true, false);
		addGraphic( sky );

		surface1 = new Backdrop( "graphics/surface.png", true, false);
		surface1.y = 110;
		addGraphic(surface1);

		surface2 = new Backdrop( "graphics/surface.png", true, false);
		surface2.y = 110;
		addGraphic(surface2);
		

		lighting = new Lighting();
		player = new Player(100, 100);
		add( player );

		// HUD
		cargo = new Text("Cargo 0/10\nValue 0", 5, 20);
		money = new Text("0", HXP.width-105, 20, 100, 50, { align:TextFormatAlign.RIGHT });
		cargo.scrollX = cargo.scrollY = money.scrollX = money.scrollY = 0;
	}

	public function cellLoaded(x:Float, y:Float) {
		var cx:Int = Std.int(Math.floor(x / Cell.size));
		var cy:Int = Std.int(Math.floor(y / Cell.size));
		var key:String = cx + "." + cy;
		var cell:Cell = cells.get(key);
		return cell!=null && cell.loaded;
	}

	public override function update() {
		super.update();

		if(title.world!=null && Input.pressed(Key.ANY)) remove(title);

		camera.x = player.x - HXP.halfWidth;
		camera.y = player.y - HXP.halfHeight;
		if(camera.y<0) camera.y=0;

		surface1.x -= 1;
		surface2.x += 1;

		// Load cells
		var cx:Int = Std.int(Math.floor((camera.x + HXP.halfWidth) / Cell.size));
		var cy:Int = Std.int(Math.floor((camera.y + HXP.halfHeight) / Cell.size));
		for(x in (cx-2) ... (cx+3)) {
			for(y in (cy-2) ... (cy+3)) {
				var key:String = x+"."+y;
				var cell:Cell = cells.get(key);
				if(cell == null) {
					cell = new Cell(x, y);
					cells.set(key, cell);
				}

				// Load or unload
				if(x<cx-1 || x>cx+1 || y<cy-1 || y>cy+1) {
					if(cell.loaded) cell.unload();
				} else if(!cell.loaded) {
					cell.load();
					//cell.loadFish();
				}
			}
		}

		// Load fish?
		if(fish<30) {
			var fx: Float = HXP.random * 1000 + camera.x;
			var fy: Float = HXP.random * 1000 + camera.y;
			if(fx>camera.x-HXP.width && fx<=camera.x) fx -= HXP.width;
			if(fx<camera.x+HXP.width && fx>=camera.x) fx += HXP.width;
			if(fy<200) fy = 400 - fy;
			
			var type:String = "fish";
			if(player.y > 500 && HXP.random<0.2)  type = "shark";
			if(player.y > 2000 && HXP.random<0.1) type = "angler";
			if(player.y > 300 && HXP.random<0.1)  type = "jelly";

			add( new Fish(fx, fy, type) );
		}


		// Update HUD
		cargo.text = "Cargo " + player.chests + "/" + player.capacity + "\nValue " + player.value + "\nHull " + player.hull;
		money.text = "Bank\n" + player.money;
	}

	public override function render() {
		// Clear
		Draw.rect(Std.int(camera.x), Std.int(camera.y), HXP.width, HXP.height, 0x2e4a9b);


		super.render();

		lighting.render();

		// Not in scene to avoid lighting
		Draw.graphic(cargo);
		Draw.graphic(money);

	}

}

