import com.haxepunk.HXP;
import com.haxepunk.Entity;
import com.haxepunk.graphics.Image;

class Fish extends Entity {

	public var image:Image;
	public var light: Light;
	public var vx: Float;
	public var vy: Float;

	public var turn: Int = 0;
	public var seek: Float = 0;
	public var moveSpeed: Float = 0;
	public var seekSpeed: Float = 0;

	public var damage: Float = 0;

	
	public function new(x:Float, y:Float, type: String) {
		image = new Image("graphics/" + type + ".png");
		image.centerOrigin();
		super(x, y, image);
		setHitboxTo(image);
		graphic = image;


		if(type=="fish") {
			moveSpeed = 3;
			seek = 0;
			damage = 2;
		} else if(type=="shark") {
			moveSpeed = 2;
			seekSpeed = 2;
			seek = 140;
			damage = 4;
		} else if(type=="jelly") {
			moveSpeed = 1;
			light = new Light(20);
			light.brightness = 0.3;
			light.oy = -20;
			damage = 10;
		} else if(type=="angler") {
			seekSpeed = 4;
			seek = 200;
			light = new Light(30);
			light.brightness = 0.5;
			light.oy = -20;
			light.ox = 60;
			damage = 20;
		}

		vx = moveSpeed;
		vy = 0;
		seek *= seek;
		this.type = "fish";
	}

	public override function added() {
		++cast(world, MainScene).fish;
		if(light!=null) cast(world, MainScene).lighting.addLight( light );
	}

	public override function removed() {
		--cast(world, MainScene).fish;
		if(light!=null) cast(world, MainScene).lighting.removeLight( light );
	}

	public override function update() {
		if(collide("solid", x, y)!=null) {
			world.remove(this);
			return;
		}

		var lx: Float = x;
		moveBy(vx, vy, "solid");
		if(vx!=0 && x==lx) vx = -vx;

		if(--turn<0) {
			vx = -vx;
			turn = 50 + HXP.rand(800);
		}

		if(seek>0) {
			var p:Entity = cast(world, MainScene).player;
			var d:Float = HXP.distanceSquared( p.x, p.y, x, y);
			if(d<seek && p.visible && d > 1000) {
				d = Math.sqrt(d) / seekSpeed;
				vx = (p.x-x) / d;
				vy = (p.y-y) / d; 
			}
			else {
				vy = 0;
				vx = vx>0? moveSpeed: -moveSpeed;
			}
		}

		if(light!=null) {
			light.x = x;
			light.y = y;
			if(image.flipped != (light.ox<0)) light.ox = -light.ox;
		}

		// Rotation
		image.flipped = vx<0;
		//image.angle = Math.atan2(vy, 10) * 180 / Math.PI * (image.flipped? 1: -1);

		// Delete
		if(x<world.camera.x-1000 || x>world.camera.x+1000 || y<world.camera.y-1000 || y>world.camera.y+1000) {
			world.remove(this);
			return;
		}
	}
	

}


