import com.haxepunk.Entity;
import com.haxepunk.graphics.Image;
import com.haxepunk.graphics.Emitter;
import com.haxepunk.utils.*;
import com.haxepunk.Sfx;


class Player extends Entity {

	public var speed: Float = 0.8;
	public var mass:  Float = 1.0;
	public var resist: Float = 0.85;
	public var hull: Float = 100;

	public var invincible: Int = 0;

	public var capacity: Int = 10;
	public var chests:   Int = 0;
	public var value:    Int = 0;

	public var money: Int = 0;

	public var vx: Float = 0;
	public var vy: Float = 0;
	private var lx: Float = 0; // Stuck?

	
	private var image: Image;
	private var particles: Emitter;
	private var broken: Emitter;
	private var light: Light;

	private var explode: Sfx;
	private var pickup: Sfx;
	private var save: Sfx;
	private var ouch: Sfx;

	
	public function new(x:Int, y:Int) {
		image = new Image("graphics/submarine.png");
		super(x, y, image);
		image.centerOrigin();

		particles = new Emitter("graphics/particles.png", 4, 4);
		particles.newType("stuff", [ 0,1,2,3]);
		particles.setMotion("stuff", 0, 40, 3, 24, 10, 4, Ease.expoOut);

		light = new Light(120, x, y);

		broken = new Emitter("graphics/parts.png", 16, 16);
		for(i in 0...9) {
			broken.newType(Std.string(i), [i]);
			broken.setMotion(Std.string(i), 0, 80, 10, 360, 20, 4, Ease.expoOut);
		}
		explode = new Sfx("audio/die.mp3");
		pickup = new Sfx("audio/coin.mp3");
		save = new Sfx("audio/save.mp3");
		ouch = new Sfx("audio/ouch.mp3");


		Input.define("up",    [ Key.W, Key.UP ]);
		Input.define("down",  [ Key.S, Key.DOWN ]);
		Input.define("left",  [ Key.A, Key.LEFT ]);
		Input.define("right", [ Key.D, Key.RIGHT ]);

		setHitboxTo(image);
		setHitbox(image.width-10, image.height-10, Std.int((image.width-10)/2), Std.int((image.height-10)/2));

	}

	public override function added() {
		world.addGraphic(particles);
		world.addGraphic(broken);
		cast(world, MainScene).lighting.addLight( light );
	}


	public function spawn() {
		x = 0;
		y = -100;
		vx = 0;
		vy = 0;
		invincible = 10;
		hull = 100;
		chests = 0;
		value = 0;
		visible = true;
		light.brightness = 1.0;

		money -= 100;
		pickup.play();

	}

	public override function update() {
		
		if(hull==0) {
			light.brightness *= 0.98;
			if(--invincible<0 && Input.pressed(Key.ANY)) spawn();
			return;
		}
	

		var moveX:Float=0, moveY:Float=0;
		if(Input.check("up"))    moveY -= 1.0;
		if(Input.check("down"))  moveY += 1.0;
		if(Input.check("left"))  moveX -= 1.0;
		if(Input.check("right")) moveX += 1.0;

		if(y<120) {
			vy += 0.5;
			if(vy>20) vy=20;
			if(lx==x && Math.abs(vx)>1) vx = vx>0? -3: 3;
		} else {
			vx *= resist;
			vy *= resist;
			vx += moveX * speed;
			vy += moveY * speed;
		}
		if(moveX != 0) image.flipped = moveX < 0;


		image.angle = Math.atan2(vy, 10) * 180 / Math.PI * (image.flipped? 1: -1);

		// Get offset vector for attachments
		var angle: Float = image.angle;
		if( !image.flipped) angle = 180 - angle;
		var ra: Float = angle * Math.PI/180;
		var ca: Float = Math.cos(ra), sa:Float = Math.sin(ra);
		var ex: Float, ey: Float;
		if(image.flipped) {
			ex =  ca * image.width/2;
			ey = -sa * image.width/2;
		} else {
			ex = ca * image.width/2;
			ey = sa * image.width/2;
		}

		if(y>120 && (moveX!=0 || moveY !=0)) {
			particles.setMotion("stuff", angle, 40, 3, 24, 10, 4, Ease.expoOut);
			particles.emit("stuff", x+ex, y+ey+5);
		}

		// Load treasure
		var treasure: Entity = collide("treasure", x, y);
		if(treasure!=null && chests<capacity) {
			world.remove(treasure);
			chests++;
			value += 1 + Math.floor(treasure.y * treasure.y / 300000);
			pickup.play();
		}
		if(y<120 && value>0) {
			save.play(0.5);
			money += value;
			value = 0;
			chests = 0;
		}

		if(invincible>0) --invincible;
		else if(hull>0) {
			var fish: Entity = collide("fish", x, y);
			if(fish!=null) {
				hull -= cast(fish, Fish).damage;
				invincible = 5;
				ouch.play(0.3);
			}
			if(hull<=0) {
				hull=0;
				invincible = 30; // Death timer
				visible = false;
				// Particles
				for(i in 0...9) broken.emit(Std.string(i), x, y);
				explode.play();
			}
		}


		lx = x;
		var ly:Float = y;
		moveBy(vx, vy, "solid");
		if(moveX!=0 && x==lx) {
			if(collide("solid", x, y+1)==null) y+=1;
			else if(collide("solid", x, y-1)==null) y-=1;
		}
		if(moveY!=0 && y==ly && x==lx) {
			if(collide("solid", x+1, y)==null) x+=1;
			else if(collide("solid", x-1, y)==null) x-=1;
		}


		light.x = x - ex;
		light.y = y - ey;

	}
	

}


